import logging
from typing import Optional

from vocode.streaming.models.synthesizer import SynthesizerConfig, SynthesizerType
from vocode.streaming.synthesizer.azure_synthesizer import AzureSynthesizer
from vocode.streaming.synthesizer.eleven_labs_synthesizer import ElevenLabsSynthesizer
from vocode.streaming.synthesizer.google_synthesizer import GoogleSynthesizer
from vocode.streaming.synthesizer.gtts_synthesizer import GTTSSynthesizer
from vocode.streaming.synthesizer.play_ht_synthesizer import PlayHtSynthesizer
from vocode.streaming.synthesizer.rime_synthesizer import RimeSynthesizer
from vocode.streaming.synthesizer.stream_elements_synthesizer import StreamElementsSynthesizer


class SynthesizerFactory:
    def create_synthesizer(
        self,
        synthesizer_config: SynthesizerConfig,
        logger: Optional[logging.Logger] = None,
    ):
        if synthesizer_config.type == SynthesizerType.GOOGLE:
            return GoogleSynthesizer(synthesizer_config, logger=logger)
        elif synthesizer_config.type == SynthesizerType.AZURE:
            return AzureSynthesizer(synthesizer_config, logger=logger)
        elif synthesizer_config.type == SynthesizerType.ELEVEN_LABS:
            return ElevenLabsSynthesizer(synthesizer_config, logger=logger)
        elif synthesizer_config.type == SynthesizerType.PLAY_HT:
            return PlayHtSynthesizer(synthesizer_config, logger=logger)
        elif synthesizer_config.type == SynthesizerType.RIME:
            return RimeSynthesizer(synthesizer_config, logger=logger)
        elif synthesizer_config.type == SynthesizerType.GTTS:
            return GTTSSynthesizer(synthesizer_config, logger=logger)
        elif synthesizer_config.type == SynthesizerType.STREAM_ELEMENTS:
            return StreamElementsSynthesizer(synthesizer_config, logger=logger)
        else:
            raise Exception("Invalid synthesizer config")



import io
from typing import Optional
from pydub import AudioSegment
import requests
from vocode import getenv
from vocode.turn_based.synthesizer.base_synthesizer import BaseSynthesizer

ELEVEN_LABS_BASE_URL = "https://api.elevenlabs.io/v1/"


class ElevenLabsSynthesizer(BaseSynthesizer):
    def __init__(
        self,
        voice_id: str,
        stability: Optional[float] = None,
        similarity_boost: Optional[float] = None,
        api_key: Optional[str] = None,
    ):
        self.voice_id = voice_id
        self.api_key = getenv("ELEVEN_LABS_API_KEY", api_key)
        self.validate_stability_and_similarity_boost(stability, similarity_boost)
        self.stability = stability
        self.similarity_boost = similarity_boost

    def validate_stability_and_similarity_boost(
        self, stability: Optional[float], similarity_boost: Optional[float]
    ) -> None:
        if (stability is None) != (similarity_boost is None):
            raise ValueError(
                "Both stability and similarity_boost must be set or not set."
            )

    def synthesize(self, text: str) -> AudioSegment:
        url = ELEVEN_LABS_BASE_URL + f"text-to-speech/{self.voice_id}"
        headers = {"xi-api-key": self.api_key, "voice_id": self.voice_id}
        body = {
            "text": text,
        }

        if self.stability is not None and self.similarity_boost is not None:
            body["voice_settings"] = {
                "stability": self.stability,
                "similarity_boost": self.similarity_boost,
            }

        response = requests.post(url, headers=headers, json=body)
        assert response.ok, response.text
        return AudioSegment.from_mp3(io.BytesIO(response.content))



from typing import Generator, Optional, Tuple
from vocode.streaming.agent.base_agent import BaseAgent


class EchoAgent(BaseAgent):
    def respond(
        self,
        human_input,
        is_interrupt: bool = False,
        conversation_id: Optional[str] = None,
    ) -> Tuple[str, bool]:
        return human_input, False

    def generate_response(
        self,
        human_input,
        is_interrupt: bool = False,
        conversation_id: Optional[str] = None,
    ) -> Generator:
        yield human_input

    def update_last_bot_message_on_cut_off(self, message: str):
        pass



from fastapi import WebSocket
from vocode.streaming.models.audio_encoding import AudioEncoding
from vocode.streaming.output_device.base_output_device import BaseOutputDevice
from vocode.streaming.models.websocket import AudioMessage


class WebsocketOutputDevice(BaseOutputDevice):
    def __init__(
        self, ws: WebSocket, sampling_rate: int, audio_encoding: AudioEncoding
    ):
        super().__init__(sampling_rate, audio_encoding)
        self.ws = ws
        self.active = True

    def mark_closed(self):
        self.active = False

    async def send_async(self, chunk: bytes):
        if self.active:
            audio_message = AudioMessage.from_bytes(chunk)
            await self.ws.send_text(audio_message.json())

    async def maybe_send_mark_async(self, message):
        pass



import audioop
import logging
from pydub import AudioSegment
import base64
from vocode import getenv
from vocode.streaming.agent.bot_sentiment_analyser import BotSentiment
from vocode.streaming.models.audio_encoding import AudioEncoding
from vocode.streaming.models.message import BaseMessage

from .base_synthesizer import BaseSynthesizer, SynthesisResult, encode_as_wav
from typing import Any, Optional
import os
import io
from dotenv import load_dotenv
import requests

from vocode.streaming.models.synthesizer import RimeSynthesizerConfig

load_dotenv()

# https://rime.ai/docs/quickstart
RIME_SAMPLING_RATE = 22050
RIME_BASE_URL = "https://rjmopratfrdjgmfmaios.functions.supabase.co/rime-tts"


class RimeSynthesizer(BaseSynthesizer):
    def __init__(
        self, config: RimeSynthesizerConfig, logger: Optional[logging.Logger] = None
    ):
        super().__init__(config)
        self.api_key = getenv("RIME_API_KEY")
        self.speaker = config.speaker

    def create_speech(
        self,
        message: BaseMessage,
        chunk_size: int,
        bot_sentiment: Optional[BotSentiment] = None,
    ) -> SynthesisResult:
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }

        body = {
            "text": message.text,
            "speaker": self.speaker,
        }
        response = requests.post(RIME_BASE_URL, headers=headers, json=body, timeout=5)
        if not response.ok:
            raise Exception(f"Rime API error: {response.status_code}, {response.text}")
        audio_file = io.BytesIO(base64.b64decode(response.json().get("audioContent")))

        return self.create_synthesis_result_from_wav(
            file=audio_file, message=message, chunk_size=chunk_size
        )



import logging
from typing import Optional
from vocode.turn_based.agent.base_agent import BaseAgent
from pyllamacpp.model import Model


class LlamaCPPAgent(BaseAgent):
    SENTENCE_ENDINGS = [".", "!", "?"]

    DEFAULT_PROMPT_TEMPLATE = "{history}\nHuman: {human_input}\nAI:"

    def __init__(
        self,
        model_path: str,
        system_prompt: str,
        initial_message: Optional[str] = None,
        logger: logging.Logger = None,
    ):
        super().__init__(initial_message)
        self.prompt_template = f"{system_prompt}\n\n{self.DEFAULT_PROMPT_TEMPLATE}"
        self.logger = logger or logging.getLogger(__name__)
        self.memory = [f"AI: {initial_message}"] if initial_message else []
        self.llm = Model(ggml_model=model_path, log_level=logging.NOTSET)

    def create_prompt(self, human_input):
        history = "\n".join(self.memory[-5:])
        return self.prompt_template.format(history=history, human_input=human_input)

    def get_memory_entry(self, human_input, response):
        return f"Human: {human_input}\nAI: {response}"

    def respond(
        self,
        human_input,
    ) -> tuple[str, bool]:
        self.logger.debug("LLM responding to human input")
        prompt = self.create_prompt(human_input)
        response = self.llm.generate(prompt, n_predict=55)[(len(prompt) + 1):]
        if "Human:" in response:
            response = response.split("Human:")[0]
        else:
            for sentence_ending in self.SENTENCE_ENDINGS:
                if sentence_ending in response:
                    response = response.split(sentence_ending)[0]
                    break
        self.memory.append(self.get_memory_entry(human_input, response))
        self.logger.debug(f"LLM response: {response}")
        return response, False


if __name__ == "__main__":
    chat_responder = LlamaCPPAgent(
        system_prompt="The AI is having a pleasant conversation about life.",
        model_path="/Users/ajayraj/src/gpt4all/gpt4all-lora-quantized-converted.bin",
    )
    while True:
        response = chat_responder.respond(input("Human: "))[0]
        print(f"AI: {response}")
